/********************************************************
IAR+SPL
Chip type: STM8S003K3 (8...16MHz HSE)
Project  : 12V acid battery cap. tester
Date     : Dec, 2016
*********************************************************/
#include "stm8s.h"
#include "iostm003.c"
#include <stdbool.h>
#include <stdlib.h>
#include "bitset.c"
#include "defines.c"
#include "periph_init.c"
#include "sswork.c"
#include "kwork.c"
#include "adcwork.c"
#include "ahcalc.c"
#include "beepwork.c"

volatile u16 tim0,//msc
             tim1,//systick
             tim2,//kb
             tim3;//beep

volatile u32 m_timer;
volatile bool m_trg;


void devcon();

extern "C"
{
 INTERRUPT_HANDLER(TIM2_OVF, 13)
 {
  static bool strg;
  if(strg)display_data(),strg=0;
  else kb.scan_keys(),strg=1;

  if(tim0)tim0--;
  if(tim1)tim1--;
  if(tim2)tim2--;
  if(tim3)tim3--;
  
  if(m_trg)m_timer++;
  
  TIM2->SR1 = 0;
 } 
}



void main()
{
 cfg.gpio();
 cfg.ee();    
 cfg.clk();
 cfg.t2(1000);
 cfg.adc();
 detect_7seg_polarity();
 
 asm("rim");

 m:
 {
  devcon();

 }
 goto m; 
}

void devcon()  
{
 static bool strg=0;
 static enum {start,inms,set_lt,ready}stage=start;
 static u8 l_st = eep.LV;
 static bool kb_trg=0, r_trg=0, se_trg=0;;
 
 if(kb.btn[0]==kb.rls && kb.btn[1]==kb.rls)kb_trg=0;
 
 switch(stage)
 {
  case start:
  {
   if(!strg)
   {
    strg=1;
    tim0=300;
    beep(4,1);
    while(tim0);
    beep(4,0);
    convert_data(calc_Vbat()/100); 
    dpp=1;
    dp_trg = 1;
   }
   
   if(kb.btn[0]==kb.prs || kb.btn[1]==kb.prs) se_trg=1;
   else if(kb.btn[0]==kb.hold || kb.btn[1]==kb.hold)
   {
    se_trg=0;
    kb_trg=1;
    stage = set_lt;
   }
   else if(kb.btn[0]==kb.rls && kb.btn[1]==kb.rls && se_trg)
   {
    se_trg=0;
    tim0 = 1000;
    r_trg = 1;
    dp_trg=0;
    dbuf[2]='r';dbuf[1]='u';dbuf[0]='n';
   }
   
   if(r_trg&&!tim0)
   {
    r_trg = 0;
    if(calc_Vbat()>eep.Ltrs) stage = inms;
    else stage = ready;
   }
   
   if(stage!=start)
   {
    strg=0;
    m_trg = 0;
    m_timer = 0;
   }
  }
  break;
  
  case inms:
  {
   if(!strg)
   {
    strg=1;
    dp_trg=1;
    
    tim0=500;
    beep(2,1);
    while(tim0);
    beep(2,0);
   }


   if((kb.btn[0]==kb.hold || kb.btn[1]==kb.hold)&&!kb_trg) 
   { 
    kb_trg=1;
    stage = set_lt;
   }    
  
   if(!tim1)
   {
    static bool d_trg=0;
    
    tim1=3000;
    d_trg^=1;    
    
    if(d_trg)convert_ahm(m_timer,dbuf,dpp,l_st);
    else
    {
     load_1=load_2=m_trg=0;
     u16 vtmp = calc_Vbat();
     if(l_st==3)load_1=load_2=1;
     else if(l_st==2)load_2=1;
     else load_1=1; 
     m_trg=1;
     
     convert_data(vtmp/100);
     dpp=1;
     if(vtmp<eep.Ltrs) stage = ready;
    }
   }
   
   if(stage!=inms)
   {
    strg=0;
   }
  }
  break;
  
  case ready:
  {
   static bool snd_trg;   
   if(!strg)
   {
    strg=1;
    load_1=load_2=0;
    m_trg=0;
    snd_trg=1;
    dp_trg=1;
   }
   
   if(!kb_trg)
   {
    if(kb.btn[0]==kb.prs || kb.btn[1]==kb.prs) se_trg=1;
    else if(kb.btn[0]==kb.hold || kb.btn[1]==kb.hold)
    {
     se_trg=0;
     stage = inms; 
    }
    else if(kb.btn[0]==kb.rls && kb.btn[1]==kb.rls && se_trg)
    {
     se_trg=0;
     snd_trg^=1;
    }    
   }
  
   if(!tim1)
   {
    static bool d_trg=0;
    d_trg^=1;    
    
    if(d_trg)
    {
     tim1=4000;     
     convert_ahm(m_timer,dbuf,dpp,l_st);
     if(snd_trg)beep(1,1);
    }
    else
    {
     tim1=1000;     
     u16 vtmp = calc_Vbat();  
     convert_data(vtmp/100);
     dpp=1;
     beep(1,0);
    }
   }   
  

   if(stage!=ready)
   {
    strg=0; 
   }
  }
  break;

  case set_lt:
  {
   if(!strg)
   {
    strg=1;
    m_trg = 0;
    tim0 = 1000;
    dbuf[2]='S';dbuf[1]='E';dbuf[0]='t';
    dp_trg=0;
    while(tim0);
    tim0 = 10000;    
   }
  
   static u16 lts_tmp=eep.Ltrs;
   u16 itmp=lts_tmp;
   
   kb.input(lts_tmp,(u16)6900,(u16)14100,(u8)100,(u16)100);
   if(lts_tmp!=itmp)
   {
    tim0=8000;
    convert_data(itmp/100);
    dp_trg=1;
    dpp=1;
   }
   
   if(!tim0)
   {
    stage=inms;
    if(eep.Ltrs!=lts_tmp)
    {
     FLASH_Unlock(FLASH_MEMTYPE_DATA);
     eep.Ltrs = lts_tmp;
     FLASH_Lock(FLASH_MEMTYPE_DATA);
    }  
    tim0 = 1000;
    dbuf[2]='c';dbuf[1]='o';dbuf[0]='n'; 
    dp_trg=0;    
    while(tim0);    
   }

   if(stage!=set_lt)
   {
    strg=0; 
   }
  }
  break;  
 }
}

